; Scroll.bb
;
; by Mike Wiering (mike@wieringsoftware.nl)
;
; This example shows how Blitz Basic can be used together with Tile Studio
; (http://tilestudio.sourceforge.net/).
;
; Press cursor keys to scroll, Esc to quit


AppTitle "Tile Studio Blitz Basic Scroll Demo"

Const SCREEN_WIDTH  = 640
Const SCREEN_HEIGHT = 480

Graphics SCREEN_WIDTH, SCREEN_HEIGHT
SetBuffer BackBuffer()

Global xscroll%, yscroll%, xspeed%, yspeed%
Global currentmap%


Include "ScrollDemo_include.bb"


Global time% = 0

; calculate length of all tile sequences
MaxSeq% = 1
Dim BlocksSeqLen (MaxSeq)
For i% = 1 To MaxSeq
  s$ = BlocksSequences$ (i)
  l% = 0
  For j = 3 To Len (s$) Step 2
    l = l + 1 + Asc (Mid$ (s$, j))
  Next
  BlocksSeqLen (i) = l
Next

xscroll = 0
yscroll = 0

xspeed = 1
yspeed = 1

maxxscroll% = MapWidth (LAYER_Blocks_Level1) * TileWidth (LAYER_Blocks_Level1) - SCREEN_WIDTH
maxyscroll% = MapHeight (LAYER_Blocks_Level1) * TileHeight (LAYER_Blocks_Level1) - SCREEN_HEIGHT

; scroll through the area
While Not KeyDown (1)  ; Esc

  If KeyDown (200) Then yspeed = yspeed - 1  ; Up
  If KeyDown (203) Then xspeed = xspeed - 1  ; Left
  If KeyDown (205) Then xspeed = xspeed + 1  ; Right
  If KeyDown (208) Then yspeed = yspeed + 1  ; Down

  xscroll = xscroll + xspeed
  yscroll = yscroll + yspeed

  If xscroll < 0 Then xscroll = 0 : xspeed = 0
  If yscroll < 0 Then yscroll = 0 : yspeed = 0
  If xscroll > maxxscroll Then xscroll = maxxscroll : xspeed = 0
  If yscroll > maxyscroll Then yscroll = maxyscroll : yspeed = 0
 
  RenderMap (LAYER_BackGr_Map1, xscroll / 3, yscroll / 3)
  RenderMap (LAYER_Blocks_Level1, xscroll, yscroll)

  time = time + 1  
  Flip
Wend
End


; get current tile from a sequence
Function GetSeqTile (n%, time%)
  time = time Mod BlocksSeqLen (n)
  s$ = BlocksSequences$ (n)
  For i% = 3 To Len (s$) Step 2
    If Asc (Mid$ (s$, i)) + 1 > time Then
      Return Asc (Mid$ (s$, i - 1)) - 1
    Else
      time = time - (Asc (Mid$ (s$, i)) + 1)
    End If
  Next
End Function


; draw the map
Function RenderMap (map%, x_offset%, y_offset%)
  ty% = y_offset / TileHeight (map)
  sy% = -(y_offset Mod TileHeight (map))
  While sy < SCREEN_HEIGHT
    ty% = ty Mod MapHeight (map)
    tx% = x_offset / TileWidth (map)
    sx% = -(x_offset Mod TileWidth (map))
    While sx < SCREEN_WIDTH 
      tile% = GetTile (map, tx Mod MapWidth (map), ty Mod MapHeight (map))
      If tile < -1 Then
        seq = -(tile + 1)
        tile = GetSeqTile (seq, time)
      End If
      If tile >= 0 Then
        DrawTile map, sx, sy, tile
      End If
      tx = tx + 1
      sx = sx + TileWidth (map)
    Wend
    ty = ty + 1
    sy = sy + TileHeight (map)
  Wend
End Function
